class ApplicationController < ActionController::Base
  rescue_from Seahorse::Client::NetworkingError, with: :handle_s3_networking_error
  helper Openseadragon::OpenseadragonHelper
  # Adds a few additional behaviors into the application controller
  include Blacklight::Controller
  include Hydra::Controller::ControllerBehavior

  # Adds Hyrax behaviors into the application controller
  include Hyrax::Controller
  include Hyrax::ThemedLayoutController
  with_themed_layout '1_column'
  before_action :check_agreement_acceptance

  private

  def handle_s3_networking_error(exception)
    respond_to do |format|
      format.html do
        render 'errors/error_message', locals: { heading: "Connectivity Issue", description: t('hyrax.uploads.js_templates.options.messages.s3_server_error') }, status: :internal_server_error
      end
      format.json { render plain: 's3_server_error', status: 500 }
    end
  end

  def check_agreement_acceptance
    if current_user.present? && !current_user.agreement_accepted?
      redirect_to main_app.deposite_agreement_path 
    end
  end
end
