# frozen_string_literal: true
class CollectionBrandingInfo < ApplicationRecord
  attr_accessor :filename, :alt_txt
  after_initialize :set_collection_attributes

  def save(file_location, upload_file = true)
    filename = File.split(local_path).last
    role_and_filename = File.join(role, filename)
    
    if upload_file
      if ENV['USE_S3'] == 'true'
        shrine_protocol = Valkyrie::Storage::Shrine::PROTOCOL
        uri = URI(file_location)
        parts = uri.path.split('/')
        source_bucket = parts[1]
        source_key = parts[2..].join('/')
        s3 = S3StorageService.new
        s3.init_client

        obj = s3.s3_client.get_object(bucket: source_bucket, key: source_key)
        file_io = obj.body
        streamfile = storage.upload(resource: Hyrax::PcdmCollection.new(id: collection_id),
                        file: file_io,
                        original_filename: role_and_filename)

        self.local_path = streamfile.id.to_s.gsub(shrine_protocol, '')
        clear_old_s3_file(source_bucket, source_key)
      else
        storage.upload(resource: Hyrax::PcdmCollection.new(id: collection_id),
                      file: File.open(file_location),
                      original_filename: role_and_filename)

        self.local_path = find_local_filename(collection_id, role, filename)

        FileUtils.remove_file(file_location) if File.exist?(file_location)
      end
    end
    super()
  end

  def clear_old_s3_file(bucket, key)
    s3 = S3StorageService.new
    s3.init_client
    s3.delete_object(bucket, key)
  end

  def delete(_location_path = nil)
    shrine_protocol = Valkyrie::Storage::Shrine::PROTOCOL
    storage.delete(id: shrine_protocol + local_path)
    super()
  end

  def find_local_filename(collection_id, role, filename)
    local_dir = find_local_dir_name(collection_id, role)
    File.join(local_dir, filename)
  end

  def find_local_dir_name(collection_id, role)
    File.join(Hyrax.config.branding_path, collection_id.to_s, role.to_s)
  end

  private

  def set_collection_attributes
    self.alt_text ||= alt_txt || ''
    self.local_path ||= File.join(role, filename)
  end

  def storage
    Hyrax.config.branding_storage_adapter
  end
end