module ExternalServices
  extend ActiveSupport::Concern
  include ComplexHelper
  included do

    def register_ark(user:)
      return true if ENV.fetch('REGISTER_ARK', 'true') == 'false'
      ark = ArkService.new
      ark.authenticate
      object = self
      creators = object.complex_person.map{|person| "#{person.fetch(:last_name)},#{person.fetch(:first_name)}" }.compact

      metadata = {
        'creator': creators.join(';'),
        'title': object.title.first,
        'date_created': object.created_at
      }
      # url = "#{ENV['APPLICATION_URL']}/concern/crc1280_experiments/#{object.id}"
      url = Rails.application.routes.url_helpers.polymorphic_url(object, host: ENV['APPLICATION_URL'])
      resp = ark.create_ark(object.id.to_s, url, metadata)
      if resp and resp[0]
        # Save ark to metadata
        complex_identifier = { identifier: resp[2], scheme: 'ARK' }
        object.complex_identifier = object.complex_identifier + [complex_identifier]
        # mark ark private
        ark.make_ark_private(object.id)
      end

      object
    end

    def total_file_count_and_size
      s3 = S3StorageService.new
      s3.init_client
      _list_of_objects, size = s3.list_all_objects

      [_list_of_objects.count, size]
    end

    def save_metadata_as_json_in_s3
      s3 = S3StorageService.new
      s3.init_client
      bucket_name = s3.sanitise_name(ENV.fetch("S3_FILE_UPLOAD_BUCKET"))

      s3.add_content(bucket_name, "#{self.id.to_s}/system_metadata.json", self.to_json)
      s3.add_content(bucket_name, "#{self.id.to_s}/metadata.json", filter_metadata_json_file(self))
    end

    def set_format_and_size
      if self.persisted?
        s3 = S3StorageService.new
        s3.init_client
        bucket_name = s3.sanitise_name(ENV.fetch("S3_FILE_UPLOAD_BUCKET"))

        if s3.bucket_exists?(bucket_name)
          list_of_objects, total_size, list_of_format = s3.list_all_objects(bucket_name, self.id.to_s, true)
          return self if (self.format == list_of_format && self.files_size.first.to_i == total_size)
          self.files_size = [total_size.to_s]
          self.format = list_of_format
          self.date_modified = DateTime.now
        end
      end

      self
    end
  end
end